# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'logger'
require 'sqreen/kit/version'

module Sqreen
  module Kit
    class Configuration
      class << self
        attr_accessor :logger

        # http client related
        attr_accessor :ingestion_url,
                      :certificate_store,
                      :proxy_address,
                      :proxy_port,
                      :proxy_user,
                      :proxy_pass,
                      :connect_timeout,
                      :read_timeout

        # retry policy
        attr_accessor :retry_max_retries,
                      :retry_wait_s

        # authentication
        attr_accessor :session_key,
                      :api_key,
                      :app_name

        # batch collector
        attr_accessor :batch_max_delay_s,
                      :batch_flush_size,
                      :batch_max_size

        # Default value for source in signals taking a generic value
        # Something like 'sqreen:agent:ruby:1.11.1'
        # XXX: do not provide default and force user to explicitly set source?
        attr_accessor :default_source

        # @param [String] url
        def proxy_url=(url)
          if url.nil?
            self.proxy_address =
              self.proxy_port =
                self.proxy_user =
                  self.proxy_pass = nil
            return
          end

          parsed = URI.parse(url)
          raise ArgumentError, 'only http proxies are supported' unless parsed.scheme == 'http'

          self.proxy_address = parsed.host
          self.proxy_port = parsed.port
          self.proxy_user = parsed.user
          self.proxy_port = parsed.port
        end
      end

      # default values
      def self.set_defaults
        instance_variables.each do |ia|
          instance_variable_set(ia, nil)
        end
        self.logger = ::Logger.new(STDERR)
        logger.level = ::Logger::WARN
        logger.progname = 'sqreen-kit'

        self.ingestion_url = 'https://ingestion.sqreen.com/'

        self.default_source = "sqreen:kit:#{Sqreen::Kit.version}"
      end

      set_defaults
    end
  end
end
