# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/kit/signals/dto_helper'

# reference: https://github.com/sqreen/SignalsSchemas/blob/master/schemas/context/http/2020-01-01T00_00_00_000Z/schema.cue

module Sqreen
  module Kit
    module Signals
      module Context
      end
    end
  end
end

class Sqreen::Kit::Signals::Context::HttpContext
  include Sqreen::Kit::Signals::DtoHelper

  PARAMS_ATTRS = [:params_query, :params_form, :params_cookies,
                  :params_json, :params_other].freeze

  SCHEMA_VERSION = 'http/2020-01-01T00:00:00.000Z'.freeze
  add_mandatory_attrs :headers, :scheme, :verb,
                      :host, :port, :remote_ip, :remote_port, :path

  # @return [String]
  attr_accessor :rid

  # mandatory
  # @return [Array<Array<String>>] inner arrays have two values: [name, value]
  attr_accessor :headers

  # mandatory
  # @return [String]
  attr_accessor :user_agent

  # mandatory
  # @return [String]
  attr_accessor :scheme

  # mandatory
  # @param [String]
  attr_writer :verb
  def verb
    raise 'verb not set' unless defined?(@verb) && @verb
    @verb.upcase
  end

  # mandatory
  # Host header; may include the port
  # @return [String]
  attr_accessor :host

  # mandatory
  # @return [Integer|String]
  attr_accessor :port

  # mandatory
  # @return [String]
  attr_accessor :remote_ip

  # mandatory
  # @return [Integer|String]
  attr_accessor :remote_port

  # mandatory
  # XXX: Unclear is decoded or not
  # @return [String]
  attr_accessor :path

  # @return [String]
  attr_accessor :referer

  # @return [Hash]
  attr_accessor :params_query

  # @return [Hash]
  attr_accessor :params_form

  # @return [Hash]
  attr_accessor :params_cookies

  # @return [Hash]
  attr_accessor :params_json

  # @return [Hash]
  attr_accessor :params_other

  # @return [String]
  attr_accessor :endpoint

  # @return [Boolean]
  attr_accessor :reveal_replayed

  # Response status
  # @return [Integer]
  attr_accessor :status

  # Response content length
  # @return [Integer]
  attr_accessor :content_length

  # Response content type
  # @return [String]
  attr_accessor :content_type

  # Datadog trace id
  # @return [Integer]
  attr_accessor :datadog_trace_id

  # Datadog span id
  # @return [Integer]
  attr_accessor :datadog_span_id

  def to_h
    check_mandatories

    {
      request: compact_hash({
                              rid: rid,
                              headers: headers,
                              user_agent: user_agent,
                              scheme: scheme,
                              verb: verb,
                              host: host,
                              port: port,
                              remote_ip: remote_ip,
                              remote_port: remote_port,
                              path: path,
                              referer: referer,
                              parameters: compact_hash({
                                                         query: params_query,
                                                         form: params_form,
                                                         cookies: params_cookies,
                                                         json: params_json,
                                                         other: params_other,
                                                       }),
                              endpoint: endpoint,
                              # yes, this one is in CamelCase
                              isRevealReplayed: reveal_replayed,
                            }),
      response: compact_hash({
                               status: status,
                               content_type: content_type,
                               content_length: content_length,
                             }),
    }.merge!(compact_hash(
               datadog_trace_id: datadog_trace_id,
               datadog_span_id: datadog_span_id,
    ))
  end
end
