# typed: ignore

# Copyright (c) 2015 Sqreen. All Rights Reserved.
# Please refer to our terms for more information: https://www.sqreen.com/terms.html

require 'sqreen/kit/signals/dto_helper'
require 'sqreen/kit/signals/stack_trace'
require 'sqreen/kit/signals/actor'

# reference: https://github.com/sqreen/SignalsSchemas/blob/master/schemas/signals.cue

module Sqreen
  module Kit
    module Signals
      module SignalAttributes
        include DtoHelper

        # @param [Time]
        # @return [String]
        attr_accessor_time :time

        def type
          raise NotImplementedError
        end

        # @return [Actor]
        attr_accessor :actor

        # @return [String]
        attr_accessor :context_schema

        # @return [Object] specified by context schema
        attr_accessor :context

        # @return [Location]
        attr_accessor :location

        # @return [LocationInfra]
        attr_accessor :location_infra

        # @return [String]
        attr_accessor :source

        # @return [Hash{String=>Object}]
        attr_accessor :trigger

        def to_h
          {
            type: type,
          }.merge(super())
        end
      end

      class Location
        include DtoHelper

        # @param [Array<String>]
        attr_writer :stack_trace

        # @param [Exception] e
        attr_writer :exception

        # Datadog trace id
        # @return [Integer]
        attr_accessor :datadog_trace_id

        # Datadog span id
        # @return [Integer]
        attr_accessor :datadog_span_id

        def stack_trace
          if @exception
            Sqreen::Kit::StackTrace.parse(@exception)
          elsif @stack_trace
            Sqreen::Kit::StackTrace.parse_backtrace(@stack_trace)
          end
        end
      end

      class LocationInfra
        include DtoHelper

        add_mandatory_attrs :agent_version, :os_type, :hostname, :runtime_version

        # mandatory
        # @return [String]
        attr_accessor :agent_version

        # mandatory
        # @return [String]
        attr_accessor :os_type

        # mandatory
        # @return [String]
        attr_accessor :hostname

        # @return [String]
        attr_accessor :runtime_type

        # mandatory
        # @return [String]
        attr_accessor :runtime_version

        # @return [String]
        attr_accessor :libsqreen_version

        def to_h
          {
            infra: {
              agent_type: 'ruby',
            }.merge(super()),
          }
        end
      end
    end # module Signals
  end
end
